@@ -21,18 +21,18 @@ __all__ = [
 ]
 
 __author__ = "James William Pye <x@jwp.name>"
-__date__ = "2009-06-13 17:19:00-07"
+__date__ = "2009-06-13 15:56:00-07"
 
 __project__ = 'py-postgresql'
 __project_id__ = 'http://python.projects.postgresql.org'
 
-version_info = (0, 8, 2, 'final', 0)
+version_info = (0, 9, 0, 'final', 0)
 __version__ = '.'.join(map(str, version_info[:3])) + (
 	version_info[3] if version_info[3] != 'final' else ''
 )
 
 pg_iri = pg_driver = pg_param = None
-def open(iri = None):
+def open(iri = None, prompt_title = None, **kw):
 	"""
 	Create a `postgresql.api.Connection` to the server referenced by the given
 	`iri`::
@@ -44,13 +44,10 @@ def open(iri = None):
 		# Connect to 'postgres' at localhost.
 		>>> db = postgresql.open('localhost/postgres')
 
-	If the URL starts with an '&', a connector will be returned instead of a
-	connection.
-
 	(Note: "pq" is the name of the protocol used to communicate with PostgreSQL)
 	"""
 	global pg_iri, pg_driver, pg_param
-	if None in (pg_iri, pg_driver, pg_param):
+	if pg_iri is None:
 		from . import iri as pg_iri
 		from . import driver as pg_driver
 		from . import clientparameters as pg_param
@@ -68,10 +65,10 @@ def open(iri = None):
 	std_params = pg_param.collect(prompt_title = None)
 	params = pg_param.normalize(
 		list(pg_param.denormalize_parameters(std_params)) + \
-		list(pg_param.denormalize_parameters(iri_params))
+		list(pg_param.denormalize_parameters(iri_params)) + \
+		list(pg_param.denormalize_parameters(kw))
 	)
-	# Resolve the password, but never prompt.
-	pg_param.resolve_password(params, prompt_title = None)
+	pg_param.resolve_password(params)
 
 	C = pg_driver.default.fit(**params)
 	if return_connector is True:
