@@ -6,20 +6,26 @@
 import sys
 import os
 import pprint
-from struct import pack, unpack, Struct
+from struct import unpack, Struct
+from .message_types import message_types
+from .typstruct import ushort_pack, ushort_unpack, ulong_pack, ulong_unpack
+
+def pack_tuple_data(atts):
+	return b''.join([
+		b'\xff\xff\xff\xff'
+		if x is None
+		else (ulong_pack(len(x)) + x)
+		for x in atts
+	])
 
 try:
-	from .optimized import parse_tuple_message
+	from .optimized import parse_tuple_message, pack_tuple_data
 except ImportError:
 	pass
 
 StringFormat = b'\x00\x00'
 BinaryFormat = b'\x00\x01'
 
-byte = Struct("!B")
-ushort = Struct("!H")
-ulong = Struct("!L")
-
 class Message(object):
 	bytes_struct = Struct("!cL")
 	__slots__ = ()
@@ -115,8 +121,7 @@ def dict_message_repr(self):
 
 class WireMessage(Message):
 	def __init__(self, typ_data):
-		type = bytes(type)[0]
-		self.type = typ_data[0]
+		self.type = message_types[typ_data[0][0]]
 		self.data = typ_data[1]
 
 	def serialize(self):
@@ -124,14 +129,14 @@ class WireMessage(Message):
 
 	@classmethod
 	def parse(typ, data):
-		if ulong.unpack(data[1:5])[0] != len(data) - 1:
+		if ulong_unpack(data[1:5]) != len(data) - 1:
 			raise ValueError(
 				"invalid wire message where data is %d bytes and " \
 				"internal size stamp is %d bytes" %(
-					len(data), ulong.unpack(data[1:5])[0] + 1
+					len(data), ulong_unpack(data[1:5]) + 1
 				)
 			)
-		return typ((data[0], data[5:]))
+		return typ((data[0:1], data[5:]))
 
 class EmptyMessage(Message):
 	'An abstract message that is always empty'
@@ -152,7 +157,7 @@ class EmptyMessage(Message):
 
 class Notify(Message):
 	'Asynchronous notification message'
-	type = b'A'
+	type = message_types[b'A'[0]]
 	__slots__ = ('pid', 'relation', 'parameter')
 
 	def __init__(self, pid, relation, parameter = b''):
@@ -161,20 +166,20 @@ class Notify(Message):
 		self.parameter = parameter
 
 	def serialize(self):
-		return ulong.pack(self.pid) + \
+		return ulong_pack(self.pid) + \
 			self.relation + b'\x00' + \
 			self.parameter + b'\x00'
 
 	@classmethod
 	def parse(typ, data):
-		pid = ulong.unpack(data[0:4])[0]
+		pid = ulong_unpack(data[0:4])
 		relname, param, nothing = data[4:].split(b'\x00', 2)
 		return typ(pid, relname, param)
 
 class ShowOption(Message):
 	"""ShowOption(name, value)
 	GUC variable information from backend"""
-	type = b'S'
+	type = message_types[b'S'[0]]
 	__slots__ = ('name', 'value')
 
 	def __init__(self, name, value):
@@ -190,7 +195,7 @@ class ShowOption(Message):
 
 class Complete(StringMessage):
 	'Command completion message.'
-	type = b'C'
+	type = message_types[b'C'[0]]
 	__slots__ = ()
 
 	@classmethod
@@ -216,52 +221,59 @@ class Complete(StringMessage):
 
 class Null(EmptyMessage):
 	'Null command'
-	type = b'I'
+	type = message_types[b'I'[0]]
 	__slots__ = ()
 NullMessage = Message.__new__(Null)
 Null.SingleInstance = NullMessage
 
 class NoData(EmptyMessage):
 	'Null command'
-	type = b'n'
+	type = message_types[b'n'[0]]
 	__slots__ = ()
 NoDataMessage = Message.__new__(NoData)
 NoData.SingleInstance = NoDataMessage
 
 class ParseComplete(EmptyMessage):
 	'Parse reaction'
-	type = b'1'
+	type = message_types[b'1'[0]]
 	__slots__ = ()
 ParseCompleteMessage = Message.__new__(ParseComplete)
 ParseComplete.SingleInstance = ParseCompleteMessage
 
 class BindComplete(EmptyMessage):
 	'Bind reaction'
-	type = b'2'
+	type = message_types[b'2'[0]]
 	__slots__ = ()
 BindCompleteMessage = Message.__new__(BindComplete)
 BindComplete.SingleInstance = BindCompleteMessage
 
 class CloseComplete(EmptyMessage):
 	'Close statement or Portal'
-	type = b'3'
+	type = message_types[b'3'[0]]
 	__slots__ = ()
 CloseCompleteMessage = Message.__new__(CloseComplete)
 CloseComplete.SingleInstance = CloseCompleteMessage
 
 class Suspension(EmptyMessage):
 	'Portal was suspended, more tuples for reading'
-	type = b's'
+	type = message_types[b's'[0]]
 	__slots__ = ()
 SuspensionMessage = Message.__new__(Suspension)
 Suspension.SingleInstance = SuspensionMessage
 
 class Ready(Message):
 	'Ready for new query'
-	type = b'Z'
+	type = message_types[b'Z'[0]]
+	possible_states = (
+		message_types[b'I'[0]],
+		message_types[b'E'[0]],
+		message_types[b'T'[0]],
+	)
 	__slots__ = ('xact_state',)
 
 	def __init__(self, data):
+		if data not in self.possible_states:
+			raise ValueError("invalid state for Ready message: " + repr(data))
 		self.xact_state = data
 
 	def serialize(self):
@@ -269,7 +281,7 @@ class Ready(Message):
 
 class Notice(Message, dict):
 	"""Notification message"""
-	type = b'N'
+	type = message_types[b'N'[0]]
 	_dtm = {
 		b'S' : 'severity',
 		b'C' : 'code',
@@ -320,14 +332,34 @@ class Notice(Message, dict):
 				kw[typ._dtm[frag[0:1]]] = frag[1:]
 		return typ(**kw)
 
+class ClientNotice(Notice):
+	__slots__ = ()
+
+	def serialize(self):
+		raise RuntimeError("cannot serialize ClientNotice")
+
+	@classmethod
+	def parse(self):
+		raise RuntimeError("cannot parse ClientNotice")
+
 class Error(Notice):
 	"""Incoming error"""
-	type = b'E'
+	type = message_types[b'E'[0]]
 	__slots__ = ()
 
+class ClientError(Error):
+	__slots__ = ()
+
+	def serialize(self):
+		raise RuntimeError("cannot serialize ClientError")
+
+	@classmethod
+	def parse(self):
+		raise RuntimeError("cannot serialize ClientError")
+
 class FunctionResult(Message):
 	"""Function result value"""
-	type = b'V'
+	type = message_types[b'V'[0]]
 	__slots__ = ('result',)
 
 	def __init__(self, datum):
@@ -335,13 +367,13 @@ class FunctionResult(Message):
 
 	def serialize(self):
 		return self.result is None and b'\xff\xff\xff\xff' or \
-			ulong.pack(len(self.result)) + self.result
+			ulong_pack(len(self.result)) + self.result
 	
 	@classmethod
 	def parse(typ, data):
 		if data == b'\xff\xff\xff\xff':
 			return typ(None)
-		size = ulong.unpack(data[0:4])[0]
+		size = ulong_unpack(data[0:4])
 		data = data[4:]
 		if size != len(data):
 			raise ValueError(
@@ -353,15 +385,15 @@ class FunctionResult(Message):
 
 class AttributeTypes(TupleMessage):
 	"""Tuple attribute types"""
-	type = b't'
+	type = message_types[b't'[0]]
 	__slots__ = ()
 
 	def serialize(self):
-		return ushort.pack(len(self)) + b''.join([ulong.pack(x) for x in self])
+		return ushort_pack(len(self)) + b''.join([ulong_pack(x) for x in self])
 
 	@classmethod
 	def parse(typ, data):
-		ac = ushort.unpack(data[0:2])[0]
+		ac = ushort_unpack(data[0:2])
 		args = data[2:]
 		if len(args) != ac * 4:
 			raise ValueError("invalid argument type data size")
@@ -369,7 +401,7 @@ class AttributeTypes(TupleMessage):
 
 class TupleDescriptor(TupleMessage):
 	"""Tuple description"""
-	type = b'T'
+	type = message_types[b'T'[0]]
 	struct = Struct("!LhLhlh")
 	__slots__ = ()
 
@@ -377,14 +409,14 @@ class TupleDescriptor(TupleMessage):
 		return [x[0] for x in self]
 
 	def serialize(self):
-		return ushort.pack(len(self)) + b''.join([
+		return ushort_pack(len(self)) + b''.join([
 			x[0] + b'\x00' + self.struct.pack(*x[1:])
 			for x in self
 		])
 
 	@classmethod
 	def parse(typ, data):
-		ac = ushort.unpack(data[0:2])[0]
+		ac = ushort_unpack(data[0:2])
 		atts = []
 		data = data[2:]
 		ca = 0
@@ -401,18 +433,15 @@ class TupleDescriptor(TupleMessage):
 
 class Tuple(TupleMessage):
 	"""Incoming tuple"""
-	type = b'D'
+	type = message_types[b'D'[0]]
 	__slots__ = ()
 
 	def serialize(self):
-		return ushort.pack(len(self)) + b''.join([
-			x is None and b'\xff\xff\xff\xff' or ulong.pack(len(x)) + bytes(x)
-			for x in self
-		])
+		return ushort_pack(len(self)) + pack_tuple_data(self)
 
 	@classmethod
 	def parse(typ, data):
-		natts = ushort.unpack(data[0:2])[0]
+		natts = ushort_unpack(data[0:2])
 		atts = list()
 		offset = 2
 
@@ -423,7 +452,7 @@ class Tuple(TupleMessage):
 			if size == b'\xff\xff\xff\xff':
 				att = None
 			else:
-				al = ulong.unpack(size)[0]
+				al = ulong_unpack(size)
 				ao = offset
 				offset = ao + al
 				att = data[ao:offset]
@@ -437,7 +466,7 @@ class Tuple(TupleMessage):
 
 class KillInformation(Message):
 	'Backend cancellation information'
-	type = b'K'
+	type = message_types[b'K'[0]]
 	struct = Struct("!LL")
 	__slots__ = ('pid', 'key')
 
@@ -466,7 +495,7 @@ class CancelRequest(KillInformation):
 
 	def bytes(self):
 		data = self.serialize()
-		return ulong.pack(len(data) + 4) + self.serialize()
+		return ulong_pack(len(data) + 4) + self.serialize()
 
 	@classmethod
 	def parse(typ, data):
@@ -486,7 +515,7 @@ class NegotiateSSL(Message):
 
 	def bytes(self):
 		data = self.serialize()
-		return ulong.pack(len(data) + 4) + data
+		return ulong_pack(len(data) + 4) + data
 
 	def serialize(self):
 		return self.packed_version
@@ -517,7 +546,7 @@ class Startup(Message, dict):
 
 	def bytes(self):
 		data = self.serialize()
-		return ulong.pack(len(data) + 4) + data
+		return ulong_pack(len(data) + 4) + data
 
 	@classmethod
 	def parse(typ, data):
@@ -562,7 +591,7 @@ AuthNameMap = {
 
 class Authentication(Message):
 	"""Authentication(request, salt)"""
-	type = b'R'
+	type = message_types[b'R'[0]]
 	__slots__ = ('request', 'salt')
 
 	def __init__(self, request, salt):
@@ -570,46 +599,46 @@ class Authentication(Message):
 		self.salt = salt
 
 	def serialize(self):
-		return ulong.pack(self.request) + self.salt
+		return ulong_pack(self.request) + self.salt
 
 	@classmethod
 	def parse(typ, data):
-		return typ(ulong.unpack(data[0:4])[0], data[4:])
+		return typ(ulong_unpack(data[0:4]), data[4:])
 
 class Password(StringMessage):
 	'Password supplement'
-	type = b'p'
+	type = message_types[b'p'[0]]
 	__slots__ = ('data',)
 
 class Disconnect(EmptyMessage):
 	'Close the connection'
-	type = b'X'
+	type = message_types[b'X'[0]]
 	__slots__ = ()
 DisconnectMessage = Message.__new__(Disconnect)
 Disconnect.SingleInstance = DisconnectMessage
 
 class Flush(EmptyMessage):
 	'Flush'
-	type = b'H'
+	type = message_types[b'H'[0]]
 	__slots__ = ()
 FlushMessage = Message.__new__(Flush)
 Flush.SingleInstance = FlushMessage
 
 class Synchronize(EmptyMessage):
 	'Synchronize'
-	type = b'S'
+	type = message_types[b'S'[0]]
 	__slots__ = ()
 SynchronizeMessage = Message.__new__(Synchronize)
 Synchronize.SingleInstance = SynchronizeMessage
 
 class Query(StringMessage):
 	"""Execute the query with the given arguments"""
-	type = b'Q'
+	type = message_types[b'Q'[0]]
 	__slots__ = ('data',)
 
 class Parse(Message):
 	"""Parse a query with the specified argument types"""
-	type = b'P'
+	type = message_types[b'P'[0]]
 	__slots__ = ('name', 'statement', 'argtypes')
 
 	def __init__(self, name, statement, argtypes):
@@ -620,7 +649,7 @@ class Parse(Message):
 	@classmethod
 	def parse(typ, data):
 		name, statement, args = data.split(b'\x00', 2)
-		ac = ushort.unpack(args[0:2])[0]
+		ac = ushort_unpack(args[0:2])
 		args = args[2:]
 		if len(args) != ac * 4:
 			raise ValueError("invalid argument type data")
@@ -628,9 +657,9 @@ class Parse(Message):
 		return typ(name, statement, at)
 
 	def serialize(self):
-		ac = ushort.pack(len(self.argtypes))
+		ac = ushort_pack(len(self.argtypes))
 		return self.name + b'\x00' + self.statement + b'\x00' + ac + b''.join([
-			ulong.pack(x) for x in self.argtypes
+			ulong_pack(x) for x in self.argtypes
 		])
 
 class Bind(Message):
@@ -645,7 +674,7 @@ class Bind(Message):
 		rformats,  # Result formats; Sequence of BinaryFormat or StringFormat.
 	)
 	"""
-	type = b'B'
+	type = message_types[b'B'[0]]
 	__slots__ = ('name', 'statement', 'aformats', 'arguments', 'rformats')
 
 	def __init__(self, name, statement, aformats, arguments, rformats):
@@ -657,14 +686,9 @@ class Bind(Message):
 
 	def serialize(self):
 		args = self.arguments
-		ac = ushort.pack(len(args))
-		ad = b''.join([
-			b'\xff\xff\xff\xff'
-			if x is None
-			else (ulong.pack(len(x)) + x)
-			for x in args
-		])
-		rfc = ushort.pack(len(self.rformats))
+		ac = ushort_pack(len(args))
+		ad = pack_tuple_data(tuple(args))
+		rfc = ushort_pack(len(self.rformats))
 		return \
 			self.name + b'\x00' + self.statement + b'\x00' + \
 			ac + b''.join(self.aformats) + ac + ad + rfc + \
@@ -673,11 +697,11 @@ class Bind(Message):
 	@classmethod
 	def parse(typ, message_data):
 		name, statement, data = message_data.split(b'\x00', 2)
-		ac = ushort.unpack(data[:2])[0]
+		ac = ushort_unpack(data[:2])
 		offset = 2 + (2 * ac)
 		aformats = unpack(("2s" * ac), data[2:offset])
 
-		natts = ushort.unpack(data[offset:offset+2])[0]
+		natts = ushort_unpack(data[offset:offset+2])
 		args = list()
 		offset += 2
 
@@ -688,14 +712,14 @@ class Bind(Message):
 			if size == b'\xff\xff\xff\xff':
 				att = None
 			else:
-				al = ulong.unpack(size)[0]
+				al = ulong_unpack(size)
 				ao = offset
 				offset = ao + al
 				att = data[ao:offset]
 			args.append(att)
 			natts -= 1
 
-		rfc = ushort.unpack(data[offset:offset+2])[0]
+		rfc = ushort_unpack(data[offset:offset+2])
 		ao = offset + 2
 		offset = ao + (2 * rfc)
 		rformats = unpack(("2s" * rfc), data[ao:offset])
@@ -704,7 +728,7 @@ class Bind(Message):
 
 class Execute(Message):
 	"""Fetch results from the specified Portal"""
-	type = b'E'
+	type = message_types[b'E'[0]]
 	__slots__ = ('name', 'max')
 
 	def __init__(self, name, max = 0):
@@ -712,16 +736,16 @@ class Execute(Message):
 		self.max = max
 
 	def serialize(self):
-		return self.name + pack("!BL", 0, self.max)
+		return self.name + b'\x00' + ulong_pack(self.max)
 
 	@classmethod
 	def parse(typ, data):
 		name, max = data.split(b'\x00', 1)
-		return typ(name, ulong.unpack(max)[0])
+		return typ(name, ulong_unpack(max))
 
 class Describe(StringMessage):
 	"""Describe a Portal or Prepared Statement"""
-	type = b'D'
+	type = message_types[b'D'[0]]
 	__slots__ = ('data',)
 
 	def serialize(self):
@@ -738,16 +762,16 @@ class Describe(StringMessage):
 		return super().parse(data[1:])
 
 class DescribeStatement(Describe):
-	subtype = b'S'
+	subtype = message_types[b'S'[0]]
 	__slots__ = ('data',)
 
 class DescribePortal(Describe):
-	subtype = b'P'
+	subtype = message_types[b'P'[0]]
 	__slots__ = ('data',)
 
 class Close(StringMessage):
 	"""Generic Close"""
-	type = b'C'
+	type = message_types[b'C'[0]]
 	__slots__ = ()
 
 	def serialize(self):
@@ -765,17 +789,17 @@ class Close(StringMessage):
 
 class CloseStatement(Close):
 	"""Close the specified Statement"""
-	subtype = b'S'
+	subtype = message_types[b'S'[0]]
 	__slots__ = ()
 
 class ClosePortal(Close):
 	"""Close the specified Portal"""
-	subtype = b'P'
+	subtype = message_types[b'P'[0]]
 	__slots__ = ()
 
 class Function(Message):
 	"""Execute the specified function with the given arguments"""
-	type = b'F'
+	type = message_types[b'F'[0]]
 	__slots__ = ('oid', 'aformats', 'arguments', 'rformat')
 
 	def __init__(self, oid, aformats, args, rformat = StringFormat):
@@ -785,23 +809,20 @@ class Function(Message):
 		self.rformat = rformat
 
 	def serialize(self):
-		ac = ushort.pack(len(self.arguments))
-		return ulong.pack(self.oid) + \
+		ac = ushort_pack(len(self.arguments))
+		return ulong_pack(self.oid) + \
 			ac + b''.join(self.aformats) + \
-			ac + b''.join([
-				(x is None) and b'\xff\xff\xff\xff' or ulong.pack(len(x)) + x
-				for x in self.arguments
-			]) + self.rformat
+			ac + pack_tuple_data(tuple(self.arguments)) + self.rformat
 
 	@classmethod
 	def parse(typ, data):
-		oid = ulong.unpack(data[0:4])[0]
+		oid = ulong_unpack(data[0:4])
 
-		ac = ushort.unpack(data[4:6])[0]
+		ac = ushort_unpack(data[4:6])
 		offset = 6 + (2 * ac)
 		aformats = unpack(("2s" * ac), data[6:offset])
 
-		natts = ushort.unpack(data[offset:offset+2])[0]
+		natts = ushort_unpack(data[offset:offset+2])
 		args = list()
 		offset += 2
 
@@ -812,7 +833,7 @@ class Function(Message):
 			if size == b'\xff\xff\xff\xff':
 				att = None
 			else:
-				al = ulong.unpack(size)[0]
+				al = ulong_unpack(size)
 				ao = offset
 				offset = ao + al
 				att = data[ao:offset]
@@ -832,7 +853,7 @@ class CopyBegin(Message):
 
 	def serialize(self):
 		return self.struct.pack(self.format, len(self.formats)) + b''.join([
-			ushort.pack(x) for x in self.formats
+			ushort_pack(x) for x in self.formats
 		])
 
 	@classmethod
@@ -842,21 +863,21 @@ class CopyBegin(Message):
 		if len(formats_str) != natts * 2:
 			raise ValueError("number of formats and data do not match up")
 		return typ(format, [
-			ushort.unpack(formats_str[x:x+2])[0] for x in range(0, natts * 2, 2)
+			ushort_unpack(formats_str[x:x+2]) for x in range(0, natts * 2, 2)
 		])
 
 class CopyToBegin(CopyBegin):
 	"""Begin copying to"""
-	type = b'H'
+	type = message_types[b'H'[0]]
 	__slots__ = ('format', 'formats')
 
 class CopyFromBegin(CopyBegin):
 	"""Begin copying from"""
-	type = b'G'
+	type = message_types[b'G'[0]]
 	__slots__ = ('format', 'formats')
 
 class CopyData(Message):
-	type = b'd'
+	type = message_types[b'd'[0]]
 	__slots__ = ('data',)
 
 	def __init__(self, data):
@@ -870,11 +891,11 @@ class CopyData(Message):
 		return typ(data)
 
 class CopyFail(StringMessage):
-	type = b'f'
+	type = message_types[b'f'[0]]
 	__slots__ = ('data',)
 
 class CopyDone(EmptyMessage):
-	type = b'c'
+	type = message_types[b'c'[0]]
 	__slots__ = ('data',)
 CopyDoneMessage = Message.__new__(CopyDone)
 CopyDone.SingleInstance = CopyDoneMessage
