@@ -2,24 +2,31 @@
 # copyright 2009, James William Pye
 # http://python.projects.postgresql.org
 ##
+import sys
 import unittest
 import struct
 import decimal
+import socket
+import time
+from threading import Thread
+
 from ..protocol import element3 as e3
 from ..protocol import xact3 as x3
-from ..protocol import pbuffer as p_buffer_module
+from ..protocol import client3 as c3
+from ..protocol import buffer as pq_buf
 from ..protocol import typstruct as pg_typstruct
 from ..protocol import typio as pg_typio
 from .. import types as pg_types
+from ..python.socket import find_available_port, SocketFactory
 
-try:
-	from ..protocol import cbuffer as c_buffer_module
-except ImportError:
-	c_buffer_module = None
+def pair(msg):
+	return (msg.type, msg.serialize())
+def pairs(*msgseq):
+	return list(map(pair, msgseq))
 
-class buffer_test(object):
+class test_buffer(object):
 	def setUp(self):
-		self.buffer = self.bufferclass()
+		self.buffer = pq_buf.pq_message_stream()
 
 	def testMultiByteMessage(self):
 		b = self.buffer
@@ -76,13 +83,6 @@ class buffer_test(object):
 		self.failUnless(msg is not None)
 		self.failUnless(msg[0] == b'X')
 
-if c_buffer_module is not None:
-	class c_buffer(buffer_test, unittest.TestCase):
-		bufferclass = c_buffer_module.pq_message_stream
-
-class p_buffer(buffer_test, unittest.TestCase):
-	bufferclass = p_buffer_module.pq_message_stream
-
 ##
 # element3 tests
 ##
@@ -343,6 +343,235 @@ class test_xact3(unittest.TestCase):
 					rec.append(z)
 			self.failUnlessEqual(xres, tuple(rec))
 
+	def testClosing(self):
+		c = x3.Closing()
+		self.failUnlessEqual(c.messages, (e3.DisconnectMessage,))
+		c.state[1]()
+		self.failUnlessEqual(c.fatal, True)
+		self.failUnlessEqual(c.error_message.__class__, e3.ClientError)
+		self.failUnlessEqual(c.error_message['code'], '08003')
+
+	def testNegotiation(self):
+		# simple successful run
+		n = x3.Negotiation({}, b'')
+		n.state[1]()
+		n.state[1](
+			pairs(
+				e3.Notice(message = b"foobar"),
+				e3.Authentication(e3.AuthRequest_OK, b''),
+				e3.KillInformation(0,0),
+				e3.ShowOption(b'name', b'val'),
+				e3.Ready(b'I'),
+			)
+		)
+		self.failUnlessEqual(n.state, x3.Complete)
+		self.failUnlessEqual(n.last_ready.xact_state, b'I')
+		# no killinfo.. should cause protocol error...
+		n = x3.Negotiation({}, b'')
+		n.state[1]()
+		n.state[1](
+			pairs(
+				e3.Notice(message = b"foobar"),
+				e3.Authentication(e3.AuthRequest_OK, b''),
+				e3.ShowOption(b'name', b'val'),
+				e3.Ready(b'I'),
+			)
+		)
+		self.failUnlessEqual(n.state, x3.Complete)
+		self.failUnlessEqual(n.last_ready, None)
+		self.failUnlessEqual(n.error_message["code"], '08P01')
+		# killinfo twice.. must cause protocol error...
+		n = x3.Negotiation({}, b'')
+		n.state[1]()
+		n.state[1](
+			pairs(
+				e3.Notice(message = b"foobar"),
+				e3.Authentication(e3.AuthRequest_OK, b''),
+				e3.ShowOption(b'name', b'val'),
+				e3.KillInformation(0,0),
+				e3.KillInformation(0,0),
+				e3.Ready(b'I'),
+			)
+		)
+		self.failUnlessEqual(n.state, x3.Complete)
+		self.failUnlessEqual(n.last_ready, None)
+		self.failUnlessEqual(n.error_message["code"], '08P01')
+		# start with ready message..
+		n = x3.Negotiation({}, b'')
+		n.state[1]()
+		n.state[1](
+			pairs(
+				e3.Notice(message = b"foobar"),
+				e3.Ready(b'I'),
+				e3.Authentication(e3.AuthRequest_OK, b''),
+				e3.ShowOption(b'name', b'val'),
+			)
+		)
+		self.failUnlessEqual(n.state, x3.Complete)
+		self.failUnlessEqual(n.last_ready, None)
+		self.failUnlessEqual(n.error_message["code"], '08P01')
+		# unsupported authreq
+		n = x3.Negotiation({}, b'')
+		n.state[1]()
+		n.state[1](
+			pairs(
+				e3.Authentication(255, b''),
+			)
+		)
+		self.failUnlessEqual(n.state, x3.Complete)
+		self.failUnlessEqual(n.last_ready, None)
+		self.failUnlessEqual(n.error_message["code"], '--AUT')
+
+	def testInstructionAsynchook(self):
+		l = []
+		def hook(data):
+			l.append(data)
+		x = x3.Instruction([
+			e3.Query(b"NOTHING")
+		], asynchook = hook)
+		a1 = e3.Notice(message = b"m1")
+		a2 = e3.Notify(0, b'relation', b'parameter')
+		a3 = e3.ShowOption(b'optname', b'optval')
+		# "send" the query message
+		x.state[1]()
+		# "receive" the tuple
+		x.state[1]([(a1.type, a1.serialize()),])
+		a2l = [(a2.type, a2.serialize()),]
+		x.state[1](a2l)
+		# validate that the hook is not fed twice because
+		# it's the exact same message set. (later assertion will validate)
+		x.state[1](a2l)
+		x.state[1]([(a3.type, a3.serialize()),])
+		# we only care about validating that l got everything.
+		self.failUnlessEqual([a1,a2,a3], l)
+		self.failUnlessEqual(x.state[0], x3.Receiving)
+		# validate that the asynchook exception is trapped.
+		class Nee(Exception):
+			pass
+		def ehook(msg):
+			raise Nee("this should **not** be part of the summary")
+		x = x3.Instruction([
+			e3.Query(b"NOTHING")
+		], asynchook = ehook)
+		a1 = e3.Notice(message = b"m1")
+		x.state[1]()
+		import sys
+		v = None
+		def exchook(typ, val, tb):
+			nonlocal v
+			v = val
+		seh = sys.excepthook
+		sys.excepthook = exchook
+		# we only care about validating that the exchook got called.
+		x.state[1]([(a1.type, a1.serialize())])
+		sys.excepthook = seh
+		self.failUnless(isinstance(v, Nee))
+
+class test_client3(unittest.TestCase):
+	def test_catmessages(self):
+		# The optimized implementation will identify adjacent copy data, and
+		# take a more efficient route; so rigorously test the switch between the
+		# two modes.
+		self.failUnlessEqual(c3.cat_messages([]), b'')
+		self.failUnlessEqual(c3.cat_messages([b'foo']), b'd\x00\x00\x00\x07foo')
+		self.failUnlessEqual(c3.cat_messages([b'foo', b'foo']), 2*b'd\x00\x00\x00\x07foo')
+		# copy, other, copy
+		self.failUnlessEqual(c3.cat_messages([b'foo', e3.SynchronizeMessage, b'foo']),
+			b'd\x00\x00\x00\x07foo' + e3.SynchronizeMessage.bytes() + b'd\x00\x00\x00\x07foo')
+		# copy, other, copy*1000
+		self.failUnlessEqual(c3.cat_messages(1000*[b'foo', e3.SynchronizeMessage, b'foo']),
+			1000*(b'd\x00\x00\x00\x07foo' + e3.SynchronizeMessage.bytes() + b'd\x00\x00\x00\x07foo'))
+		# other, copy, copy*1000
+		self.failUnlessEqual(c3.cat_messages(1000*[e3.SynchronizeMessage, b'foo', b'foo']),
+			1000*(e3.SynchronizeMessage.bytes() + 2*b'd\x00\x00\x00\x07foo'))
+		class ThisEx(Exception):
+			pass
+		class ThatEx(Exception):
+			pass
+		class Bad(e3.Message):
+			def serialize(self):
+				raise ThisEx('foo')
+		self.failUnlessRaises(ThisEx, c3.cat_messages, [Bad()])
+		class NoType(e3.Message):
+			def serialize(self):
+				return b''
+		self.failUnlessRaises(AttributeError, c3.cat_messages, [NoType()])
+		class BadType(e3.Message):
+			type = 123
+			def serialize(self):
+				return b''
+		self.failUnlessRaises((TypeError,struct.error), c3.cat_messages, [BadType()])
+
+	def test_timeout(self):
+		portnum = find_available_port()
+		servsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+		servsock.bind(('localhost', portnum))
+		pc = c3.Connection(
+			SocketFactory(
+				(socket.AF_INET, socket.SOCK_STREAM),
+				('localhost', portnum)
+			),
+			{}
+		)
+		pc.connect(timeout = 1)
+		self.failUnlessEqual(pc.xact.fatal, True)
+		self.failUnlessEqual(type(pc.xact), x3.Negotiation)
+		servsock.close()
+
+	def test_SSL_failure(self):
+		portnum = find_available_port()
+		servsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+		servsock.bind(('localhost', portnum))
+		pc = c3.Connection(
+			SocketFactory(
+				(socket.AF_INET, socket.SOCK_STREAM),
+				('localhost', portnum)
+			),
+			{}
+		)
+		exc = None
+		def client_thread():
+			pc.connect(ssl = True)
+		client = Thread(target = client_thread)
+		client.start()
+		servsock.listen(1)
+		c, addr = servsock.accept()
+		c.send(b'S')
+		c.sendall(b'0000000000000000000000')
+		c.close()
+		servsock.close()
+		client.join()
+		self.failUnlessEqual(pc.xact.fatal, True)
+		self.failUnlessEqual(pc.xact.__class__, x3.Negotiation)
+		self.failUnlessEqual(pc.xact.error_message.__class__, e3.ClientError)
+		self.failUnless(hasattr(pc.xact, 'exception'))
+
+	def test_bad_negotiation(self):
+		portnum = find_available_port()
+		servsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+		servsock.bind(('localhost', portnum))
+		pc = c3.Connection(
+			SocketFactory(
+				(socket.AF_INET, socket.SOCK_STREAM),
+				('localhost', portnum)
+			),
+			{}
+		)
+		exc = None
+		def client_thread():
+			pc.connect()
+		client = Thread(target = client_thread)
+		client.start()
+		servsock.listen(1)
+		c, addr = servsock.accept()
+		c.close()
+		time.sleep(0.25)
+		client.join()
+		servsock.close()
+		self.failUnlessEqual(pc.xact.fatal, True)
+		self.failUnlessEqual(pc.xact.__class__, x3.Negotiation)
+		self.failUnlessEqual(pc.xact.error_message.__class__, e3.ClientError)
+		self.failUnlessEqual(pc.xact.error_message["code"], '08006')
 
 # this must pack to that, and
 # that must unpack to this
@@ -771,7 +1000,111 @@ try:
 			self.failUnlessRaises(TypeError, pt, "foo", "bar", funpass)
 			self.failUnlessRaises(TypeError, pt, (), "bar", funpass)
 			self.failUnlessRaises(TypeError, pt, "foo", (), funpass)
-			self.failUnlessRaises(ValueError, pt, (), ("foo",), funpass)
+			self.failUnlessRaises(TypeError, pt, (), ("foo",), funpass)
+
+		def test_pack_tuple_data(self):
+			pit = protocol_optimized.pack_tuple_data
+			self.failUnlessEqual(pit((None,)), b'\xff\xff\xff\xff')
+			self.failUnlessEqual(pit((None,)*2), b'\xff\xff\xff\xff'*2)
+			self.failUnlessEqual(pit((None,)*3), b'\xff\xff\xff\xff'*3)
+			self.failUnlessEqual(pit((None,b'foo')), b'\xff\xff\xff\xff\x00\x00\x00\x03foo')
+			self.failUnlessEqual(pit((None,b'')), b'\xff\xff\xff\xff\x00\x00\x00\x00')
+			self.failUnlessEqual(pit((None,b'',b'bar')), b'\xff\xff\xff\xff\x00\x00\x00\x00\x00\x00\x00\x03bar')
+			self.failUnlessRaises(TypeError, pit, 1)
+			self.failUnlessRaises(TypeError, pit, (1,))
+			self.failUnlessRaises(TypeError, pit, ("",))
+
+		def test_int2(self):
+			d = b'\x00\x01'
+			rd = b'\x01\x00'
+			s = protocol_optimized.swap_int2_unpack(d)
+			n = protocol_optimized.int2_unpack(d)
+			sd = protocol_optimized.swap_int2_pack(1)
+			nd = protocol_optimized.int2_pack(1)
+			if sys.byteorder == 'little':
+				self.failUnlessEqual(1, s)
+				self.failUnlessEqual(256, n)
+				self.failUnlessEqual(d, sd)
+				self.failUnlessEqual(rd, nd)
+			else:
+				self.failUnlessEqual(1, n)
+				self.failUnlessEqual(256, s)
+				self.failUnlessEqual(d, nd)
+				self.failUnlessEqual(rd, sd)
+			self.failUnlessRaises(OverflowError, protocol_optimized.swap_int2_pack, 2**15)
+			self.failUnlessRaises(OverflowError, protocol_optimized.int2_pack, 2**15)
+			self.failUnlessRaises(OverflowError, protocol_optimized.swap_int2_pack, (-2**15)-1)
+			self.failUnlessRaises(OverflowError, protocol_optimized.int2_pack, (-2**15)-1)
+
+		def test_int4(self):
+			d = b'\x00\x00\x00\x01'
+			rd = b'\x01\x00\x00\x00'
+			s = protocol_optimized.swap_int4_unpack(d)
+			n = protocol_optimized.int4_unpack(d)
+			sd = protocol_optimized.swap_int4_pack(1)
+			nd = protocol_optimized.int4_pack(1)
+			if sys.byteorder == 'little':
+				self.failUnlessEqual(1, s)
+				self.failUnlessEqual(16777216, n)
+				self.failUnlessEqual(d, sd)
+				self.failUnlessEqual(rd, nd)
+			else:
+				self.failUnlessEqual(1, n)
+				self.failUnlessEqual(16777216, s)
+				self.failUnlessEqual(d, nd)
+				self.failUnlessEqual(rd, sd)
+			self.failUnlessRaises(OverflowError, protocol_optimized.swap_int4_pack, 2**31)
+			self.failUnlessRaises(OverflowError, protocol_optimized.int4_pack, 2**31)
+			self.failUnlessRaises(OverflowError, protocol_optimized.swap_int4_pack, (-2**31)-1)
+			self.failUnlessRaises(OverflowError, protocol_optimized.int4_pack, (-2**31)-1)
+
+		def test_uint2(self):
+			d = b'\x00\x01'
+			rd = b'\x01\x00'
+			s = protocol_optimized.swap_uint2_unpack(d)
+			n = protocol_optimized.uint2_unpack(d)
+			sd = protocol_optimized.swap_uint2_pack(1)
+			nd = protocol_optimized.uint2_pack(1)
+			if sys.byteorder == 'little':
+				self.failUnlessEqual(1, s)
+				self.failUnlessEqual(256, n)
+				self.failUnlessEqual(d, sd)
+				self.failUnlessEqual(rd, nd)
+			else:
+				self.failUnlessEqual(1, n)
+				self.failUnlessEqual(256, s)
+				self.failUnlessEqual(d, nd)
+				self.failUnlessEqual(rd, sd)
+			self.failUnlessRaises(OverflowError, protocol_optimized.swap_uint2_pack, -1)
+			self.failUnlessRaises(OverflowError, protocol_optimized.uint2_pack, -1)
+			self.failUnlessRaises(OverflowError, protocol_optimized.swap_uint2_pack, 2**16)
+			self.failUnlessRaises(OverflowError, protocol_optimized.uint2_pack, 2**16)
+			self.failUnlessEqual(protocol_optimized.uint2_pack(2**16-1), b'\xFF\xFF')
+			self.failUnlessEqual(protocol_optimized.swap_uint2_pack(2**16-1), b'\xFF\xFF')
+
+		def test_uint4(self):
+			d = b'\x00\x00\x00\x01'
+			rd = b'\x01\x00\x00\x00'
+			s = protocol_optimized.swap_uint4_unpack(d)
+			n = protocol_optimized.uint4_unpack(d)
+			sd = protocol_optimized.swap_uint4_pack(1)
+			nd = protocol_optimized.uint4_pack(1)
+			if sys.byteorder == 'little':
+				self.failUnlessEqual(1, s)
+				self.failUnlessEqual(16777216, n)
+				self.failUnlessEqual(d, sd)
+				self.failUnlessEqual(rd, nd)
+			else:
+				self.failUnlessEqual(1, n)
+				self.failUnlessEqual(16777216, s)
+				self.failUnlessEqual(d, nd)
+				self.failUnlessEqual(rd, sd)
+			self.failUnlessRaises(OverflowError, protocol_optimized.swap_uint4_pack, -1)
+			self.failUnlessRaises(OverflowError, protocol_optimized.uint4_pack, -1)
+			self.failUnlessRaises(OverflowError, protocol_optimized.swap_uint4_pack, 2**32)
+			self.failUnlessRaises(OverflowError, protocol_optimized.uint4_pack, 2**32)
+			self.failUnlessEqual(protocol_optimized.uint4_pack(2**32-1), b'\xFF\xFF\xFF\xFF')
+			self.failUnlessEqual(protocol_optimized.swap_uint4_pack(2**32-1), b'\xFF\xFF\xFF\xFF')
 except ImportError:
 	pass
 
